@@ -16,7 +16,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: FilePicker.h,v 1.8.8.1 2004/11/13 19:56:49 chpe Exp $
+ *  $Id: FilePicker.h,v 1.10 2005/04/06 14:29:55 chpe Exp $
  */
 
 #ifndef EPHY_FILEPICKER_H
@@ -24,9 +24,9 @@
 
 #include <nsIFilePicker.h>
 #include <nsISupports.h>
-#define MOZILLA_STRICT_API
+#undef MOZILLA_INTERNAL_API
 #include <nsEmbedString.h>
-#undef MOZILLA_STRICT_API
+#define MOZILLA_INTERNAL_API 1
 
 #include "ephy-file-chooser.h"
 
