@@ -17,12 +17,10 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: ephy-node.c,v 1.30 2004/02/11 14:38:12 marco Exp $
+ *  $Id: ephy-node.c,v 1.42 2006/01/16 22:24:00 paharvey Exp $
  */
 
-#ifdef HAVE_CONFIG_H
 #include "config.h"
-#endif
 
 #include <glib.h>
 #include <glib/gi18n.h>
@@ -33,7 +31,6 @@
 #include <time.h>
 
 #include "ephy-node.h"
-#include "ephy-string.h"
 
 typedef struct
 {
@@ -42,6 +39,7 @@ typedef struct
 	EphyNodeCallback callback;
 	EphyNodeSignalType type;
 	gpointer data;
+	gboolean invalidated;
 } EphyNodeSignalData;
 
 typedef struct
@@ -56,11 +54,11 @@ typedef struct
 	guint property_id;
 } EphyNodeChange;
 
-struct EphyNode
+struct _EphyNode
 {
 	int ref_count;
 
-	gulong id;
+	guint id;
 
 	GPtrArray *properties;
 
@@ -69,6 +67,10 @@ struct EphyNode
 
 	GHashTable *signals;
 	int signal_id;
+	guint emissions;
+	guint invalidated_signals;
+	guint is_drag_source : 1;
+	guint is_drag_dest : 1;
 
 	EphyNodeDb *db;
 };
@@ -98,6 +100,8 @@ callback (long id, EphyNodeSignalData *data, gpointer *dummy)
 	ENESCData *user_data;
 	va_list valist;
 
+	if (data->invalidated) return;
+
 	user_data = (ENESCData *) dummy;
 
 	G_VA_COPY(valist, user_data->valist);
@@ -111,6 +115,16 @@ callback (long id, EphyNodeSignalData *data, gpointer *dummy)
 			data->callback (data->node, data->data);
 		break;
 
+		case EPHY_NODE_CHANGED:
+		{
+			guint property_id;
+
+			property_id = va_arg (valist, guint);
+		
+			data->callback (data->node, property_id, data->data);
+		}
+		break;
+
 		case EPHY_NODE_CHILD_ADDED:
 		{
 			EphyNode *node;
@@ -153,11 +167,21 @@ callback (long id, EphyNodeSignalData *data, gpointer *dummy)
         va_end(valist);
 }
 
+static gboolean
+remove_invalidated_signals (long id,
+			    EphyNodeSignalData *data,
+			    gpointer user_data)
+{
+	return data->invalidated;
+}
+
 static void
 ephy_node_emit_signal (EphyNode *node, EphyNodeSignalType type, ...)
 {
 	ENESCData data;
 
+	++node->emissions;
+
 	va_start (data.valist, type);
 
 	data.type = type;
@@ -167,33 +191,19 @@ ephy_node_emit_signal (EphyNode *node, EphyNodeSignalType type, ...)
 			      &data);
 
 	va_end (data.valist);
-}
 
-static void
-ephy_node_finalize (EphyNode *node)
-{
-	guint i;
+	if (G_UNLIKELY (--node->emissions == 0 && node->invalidated_signals))
+	{
+		int removed;
 
-	g_hash_table_destroy (node->signals);
-	node->signals = NULL;
+		removed = g_hash_table_foreach_remove
+				(node->signals,
+				 (GHRFunc) remove_invalidated_signals,
+				 NULL);
+		g_assert (removed == node->invalidated_signals);
 
-	for (i = 0; i < node->properties->len; i++) {
-		GValue *val;
-
-		val = g_ptr_array_index (node->properties, i);
-
-		if (val != NULL) {
-			g_value_unset (val);
-			g_free (val);
-		}
+		node->invalidated_signals = 0;
 	}
-	g_ptr_array_free (node->properties, TRUE);
-
-	g_hash_table_destroy (node->parents);
-
-	g_ptr_array_free (node->children, TRUE);
-
-	g_free (node);
 }
 
 static inline void
@@ -250,13 +260,12 @@ static void
 signal_object_weak_notify (EphyNodeSignalData *signal_data,
                            GObject *where_the_object_was)
 {
+	signal_data->data = NULL;
 	ephy_node_signal_disconnect (signal_data->node, signal_data->id);
 }
 
 static void
-unref_signal_objects (long id,
-	              EphyNodeSignalData *signal_data,
-	              EphyNode *node)
+destroy_signal_data (EphyNodeSignalData *signal_data)
 {
 	if (signal_data->data)
 	{
@@ -264,20 +273,24 @@ unref_signal_objects (long id,
 				     (GWeakNotify)signal_object_weak_notify,
 				     signal_data);
 	}
+        
+        g_free (signal_data);
 }
 
 static void
-ephy_node_dispose (EphyNode *node)
+ephy_node_destroy (EphyNode *node)
 {
 	guint i;
 
 	ephy_node_emit_signal (node, EPHY_NODE_DESTROY);
 
-	/* remove from DAG */
+        /* Remove from parents. */
 	g_hash_table_foreach (node->parents,
 			      (GHFunc) remove_child,
 			      node);
+	g_hash_table_destroy (node->parents);
 
+        /* Remove children. */
 	for (i = 0; i < node->children->len; i++) {
 		EphyNode *child;
 
@@ -285,12 +298,28 @@ ephy_node_dispose (EphyNode *node)
 
 		real_remove_child (node, child, FALSE, TRUE);
 	}
+	g_ptr_array_free (node->children, TRUE);
+        
+        /* Remove signals. */
+	g_hash_table_destroy (node->signals);
 
-	g_hash_table_foreach (node->signals,
-			      (GHFunc) unref_signal_objects,
-			      node);
-
+        /* Remove id. */
 	_ephy_node_db_remove_id (node->db, node->id);
+
+        /* Remove properties. */
+	for (i = 0; i < node->properties->len; i++) {
+		GValue *val;
+
+		val = g_ptr_array_index (node->properties, i);
+
+		if (val != NULL) {
+			g_value_unset (val);
+			g_free (val);
+		}
+	}
+	g_ptr_array_free (node->properties, TRUE);
+
+	g_free (node);
 }
 
 EphyNode *
@@ -308,7 +337,7 @@ ephy_node_new (EphyNodeDb *db)
 }
 
 EphyNode *
-ephy_node_new_with_id (EphyNodeDb *db, gulong reserved_id)
+ephy_node_new_with_id (EphyNodeDb *db, guint reserved_id)
 {
 	EphyNode *node;
 
@@ -318,7 +347,7 @@ ephy_node_new_with_id (EphyNodeDb *db, gulong reserved_id)
 
 	node = g_new0 (EphyNode, 1);
 
-	node->ref_count = 0;
+	node->ref_count = 1;
 
 	node->id = reserved_id;
 
@@ -328,16 +357,18 @@ ephy_node_new_with_id (EphyNodeDb *db, gulong reserved_id)
 
 	node->children = g_ptr_array_new ();
 
-	node->parents = g_hash_table_new_full (int_hash,
-					       int_equal,
-					       NULL,
-					       g_free);
+	node->parents = g_hash_table_new_full
+          (int_hash, int_equal, NULL, g_free);
+
+	node->signals = g_hash_table_new_full
+          (int_hash, int_equal, NULL,
+           (GDestroyNotify)destroy_signal_data);
 
-	node->signals = g_hash_table_new_full (int_hash,
-					       int_equal,
-					       NULL,
-					       g_free);
 	node->signal_id = 0;
+	node->emissions = 0;
+	node->invalidated_signals = 0;
+	node->is_drag_source = TRUE;
+	node->is_drag_dest = TRUE;
 
 	_ephy_node_db_add_id (db, reserved_id, node);
 
@@ -352,12 +383,12 @@ ephy_node_get_db (EphyNode *node)
 	return node->db;
 }
 
-long
+guint
 ephy_node_get_id (EphyNode *node)
 {
 	long ret;
 
-	g_return_val_if_fail (EPHY_IS_NODE (node), -1);
+	g_return_val_if_fail (EPHY_IS_NODE (node), G_MAXUINT);
 
 	ret = node->id;
 
@@ -380,13 +411,12 @@ ephy_node_unref (EphyNode *node)
 	node->ref_count--;
 
 	if (node->ref_count <= 0) {
-		ephy_node_dispose (node);
-		ephy_node_finalize (node);
+		ephy_node_destroy (node);
 	}
 }
 
 static void
-child_changed (gulong id,
+child_changed (guint id,
 	       EphyNodeParent *node_info,
 	       EphyNodeChange *change)
 {
@@ -439,6 +469,8 @@ ephy_node_set_property (EphyNode *node,
 	g_hash_table_foreach (node->parents,
 			      (GHFunc) child_changed,
 			      &change);
+    
+	ephy_node_emit_signal (node, EPHY_NODE_CHANGED, property_id);
 }
 
 gboolean
@@ -642,7 +674,7 @@ typedef struct
 } ForEachData;
 
 static void
-write_parent (gulong id,
+write_parent (guint id,
 	      EphyNodeParent *node_info,
 	      ForEachData* data)
 {
@@ -653,11 +685,11 @@ write_parent (gulong id,
 	 */
 	if (data->ret < 0) return;
 
-	data->ret = xmlTextWriterStartElement (writer, "parent");
+	data->ret = xmlTextWriterStartElement (writer, (const xmlChar *)"parent");
 	if (data->ret < 0) return;
 
 	data->ret = xmlTextWriterWriteFormatAttribute
-			(writer, "id", "%ld", node_info->node->id);
+			(writer, (const xmlChar *)"id", "%ld", node_info->node->id);
 	if (data->ret < 0) return;
 
 	data->ret = xmlTextWriterEndElement (writer); /* parent */
@@ -677,11 +709,11 @@ ephy_node_write_to_xml(EphyNode *node,
 	g_return_val_if_fail (writer != NULL, -1);
 
 	/* start writing the node */
-	ret = xmlTextWriterStartElement (writer, "node");
+	ret = xmlTextWriterStartElement (writer, (const xmlChar *)"node");
 	if (ret < 0) goto out;
 
 	/* write node id */
-	ret = xmlTextWriterWriteFormatAttribute (writer, "id", "%ld", node->id);
+	ret = xmlTextWriterWriteFormatAttribute (writer, (const xmlChar *)"id", "%ld", node->id);
 	if (ret < 0) goto out;
 
 	/* write node properties */
@@ -692,22 +724,25 @@ ephy_node_write_to_xml(EphyNode *node,
 		value = g_ptr_array_index (node->properties, i);
 
 		if (value == NULL) continue;
+		if (G_VALUE_TYPE (value) == G_TYPE_STRING &&
+		    g_value_get_string (value) == NULL) continue;
 
-		ret = xmlTextWriterStartElement (writer, "property");
+		ret = xmlTextWriterStartElement (writer, (const xmlChar *)"property");
 		if (ret < 0) break;
 
-		ret = xmlTextWriterWriteFormatAttribute (writer, "id", "%d", i);
+		ret = xmlTextWriterWriteFormatAttribute (writer, (const xmlChar *)"id", "%d", i);
 		if (ret < 0) break;
 
 		ret = xmlTextWriterWriteAttribute
-			(writer, "value_type", g_type_name (G_VALUE_TYPE (value)));
+			(writer, (const xmlChar *)"value_type", 
+			 (const xmlChar *)g_type_name (G_VALUE_TYPE (value)));
 		if (ret < 0) break;
 
 		switch (G_VALUE_TYPE (value))
 		{
 		case G_TYPE_STRING:
 			ret = xmlTextWriterWriteString
-				(writer, g_value_get_string (value));
+				(writer, (const xmlChar *)g_value_get_string (value));
 			break;
 		case G_TYPE_BOOLEAN:
 			ret = xmlTextWriterWriteFormatString
@@ -722,12 +757,12 @@ ephy_node_write_to_xml(EphyNode *node,
 				(writer, "%ld", g_value_get_long (value));
 			break;
 		case G_TYPE_FLOAT:
-			g_ascii_dtostr (xml_buf, sizeof (xml_buf), 
+			g_ascii_dtostr ((gchar *)xml_buf, sizeof (xml_buf), 
 					g_value_get_float (value));
 			ret = xmlTextWriterWriteString (writer, xml_buf);
 			break;
 		case G_TYPE_DOUBLE:
-			g_ascii_dtostr (xml_buf, sizeof (xml_buf),
+			g_ascii_dtostr ((gchar *)xml_buf, sizeof (xml_buf),
 					g_value_get_double (value));
 			ret = xmlTextWriterWriteString (writer, xml_buf);
 			break;
@@ -786,7 +821,7 @@ ephy_node_new_from_xml (EphyNodeDb *db, xmlNodePtr xml_node)
 {
 	EphyNode *node;
 	xmlNodePtr xml_child;
-	char *xml;
+	xmlChar *xml;
 	long id;
 
 	g_return_val_if_fail (EPHY_IS_NODE_DB (db), NULL);
@@ -794,22 +829,22 @@ ephy_node_new_from_xml (EphyNodeDb *db, xmlNodePtr xml_node)
 
 	if (ephy_node_db_is_immutable (db)) return NULL; 
 
-	xml = xmlGetProp (xml_node, "id");
+	xml = xmlGetProp (xml_node, (const xmlChar *)"id");
 	if (xml == NULL)
 		return NULL;
-	id = atol (xml);
+	id = atol ((const char *)xml);
 	xmlFree (xml);
 
 	node = ephy_node_new_with_id (db, id);
 
 	for (xml_child = xml_node->children; xml_child != NULL; xml_child = xml_child->next) {
-		if (strcmp (xml_child->name, "parent") == 0) {
+		if (strcmp ((const char *)xml_child->name, "parent") == 0) {
 			EphyNode *parent;
 			long parent_id;
 
-			xml = xmlGetProp (xml_child, "id");
+			xml = xmlGetProp (xml_child, (const xmlChar *)"id");
 			g_assert (xml != NULL);
-			parent_id = atol (xml);
+			parent_id = atol ((const char *)xml);
 			xmlFree (xml);
 
 			parent = ephy_node_db_get_node_from_id (db, parent_id);
@@ -820,17 +855,17 @@ ephy_node_new_from_xml (EphyNodeDb *db, xmlNodePtr xml_node)
 
 				ephy_node_emit_signal (parent, EPHY_NODE_CHILD_ADDED, node);
 			}
-		} else if (strcmp (xml_child->name, "property") == 0) {
+		} else if (strcmp ((const char *)xml_child->name, "property") == 0) {
 			GType value_type;
 			GValue *value;
 			int property_id;
 
-			xml = xmlGetProp (xml_child, "id");
-			property_id = atoi (xml);
+			xml = xmlGetProp (xml_child, (const xmlChar *)"id");
+			property_id = atoi ((const char *)xml);
 			xmlFree (xml);
 
-			xml = xmlGetProp (xml_child, "value_type");
-			value_type = g_type_from_name (xml);
+			xml = xmlGetProp (xml_child, (const xmlChar *)"value_type");
+			value_type = g_type_from_name ((const gchar *)xml);
 			xmlFree (xml);
 
 			xml = xmlNodeGetContent (xml_child);
@@ -840,28 +875,28 @@ ephy_node_new_from_xml (EphyNodeDb *db, xmlNodePtr xml_node)
 			switch (value_type)
 			{
 			case G_TYPE_STRING:
-				g_value_set_string (value, xml);
+				g_value_set_string (value, (const gchar *)xml);
 				break;
 			case G_TYPE_INT:
-				g_value_set_int (value, atoi (xml));
+				g_value_set_int (value, atoi ((const char *)xml));
 				break;
 			case G_TYPE_BOOLEAN:
-				g_value_set_boolean (value, atoi (xml));
+				g_value_set_boolean (value, atoi ((const char *)xml));
 				break;
 			case G_TYPE_LONG:
-				g_value_set_long (value, atol (xml));
+				g_value_set_long (value, atol ((const char *)xml));
 				break;
 			case G_TYPE_FLOAT:
-				g_value_set_float (value, g_ascii_strtod (xml, NULL));
+				g_value_set_float (value, g_ascii_strtod ((const gchar *)xml, NULL));
 				break;
 			case G_TYPE_DOUBLE:
-				g_value_set_double (value, g_ascii_strtod (xml, NULL));
+				g_value_set_double (value, g_ascii_strtod ((const gchar *)xml, NULL));
 				break;
 			case G_TYPE_POINTER:
 			{
 				EphyNode *property_node;
 
-				property_node = ephy_node_db_get_node_from_id (db, atol (xml));
+				property_node = ephy_node_db_get_node_from_id (db, atol ((const char *)xml));
 
 				g_value_set_pointer (value, property_node);
 				break;
@@ -1138,6 +1173,8 @@ ephy_node_signal_connect_object (EphyNode *node,
 	int ret;
 
 	g_return_val_if_fail (EPHY_IS_NODE (node), -1);
+	/* FIXME: */
+	g_return_val_if_fail (node->emissions == 0, -1);
 
 	signal_data = g_new0 (EphyNodeSignalData, 1);
 	signal_data->node = node;
@@ -1162,13 +1199,123 @@ ephy_node_signal_connect_object (EphyNode *node,
 	return ret;
 }
 
+static gboolean
+remove_matching_signal_data (gpointer key,
+			     EphyNodeSignalData *signal_data,
+			     EphyNodeSignalData *user_data)
+{
+	return (user_data->data == signal_data->data &&
+		user_data->type == signal_data->type &&
+		user_data->callback == signal_data->callback);           
+}
+
+static void
+invalidate_matching_signal_data (gpointer key,
+				 EphyNodeSignalData *signal_data,
+				 EphyNodeSignalData *user_data)
+{
+	if (user_data->data == signal_data->data &&
+	    user_data->type == signal_data->type &&
+	    user_data->callback == signal_data->callback &&
+	    !signal_data->invalidated)
+	{
+		signal_data->invalidated = TRUE;
+		++signal_data->node->invalidated_signals;
+	}
+}
+
+guint
+ephy_node_signal_disconnect_object (EphyNode *node,
+                                    EphyNodeSignalType type,
+                                    EphyNodeCallback callback,
+                                    GObject *object)
+{
+        EphyNodeSignalData user_data;
+
+	g_return_val_if_fail (EPHY_IS_NODE (node), 0);
+
+	user_data.callback = callback;
+	user_data.type = type;
+	user_data.data = object;
+
+	if (G_LIKELY (node->emissions == 0))
+	{
+		return g_hash_table_foreach_remove (node->signals,
+						    (GHRFunc) remove_matching_signal_data,
+						    &user_data);
+	}
+	else
+	{
+		g_hash_table_foreach (node->signals,
+				      (GHFunc) invalidate_matching_signal_data,
+				      &user_data);
+		return 0;
+	}
+}
+
 void
 ephy_node_signal_disconnect (EphyNode *node,
 			     int signal_id)
 {
 	g_return_if_fail (EPHY_IS_NODE (node));
+	g_return_if_fail (signal_id != -1);
 
-	g_hash_table_remove (node->signals,
-			     GINT_TO_POINTER (signal_id));
+	if (G_LIKELY (node->emissions == 0))
+	{
+		g_hash_table_remove (node->signals,
+				     GINT_TO_POINTER (signal_id));
+	}
+	else
+	{
+		EphyNodeSignalData *data;
+
+		data = g_hash_table_lookup (node->signals,
+					    GINT_TO_POINTER (signal_id));
+		g_return_if_fail (data != NULL);
+		g_return_if_fail (!data->invalidated);
+
+		data->invalidated = TRUE;
+		node->invalidated_signals++;
+	}
 }
 
+void
+ephy_node_set_is_drag_source (EphyNode *node,
+			      gboolean allow)
+{
+	node->is_drag_source = allow != FALSE;
+}
+
+gboolean
+ephy_node_get_is_drag_source (EphyNode *node)
+{
+	return node->is_drag_source;
+}
+
+void
+ephy_node_set_is_drag_dest (EphyNode *node,
+			    gboolean allow)
+{
+	node->is_drag_dest = allow != FALSE;
+}
+
+gboolean
+ephy_node_get_is_drag_dest (EphyNode *node)
+{
+	return node->is_drag_dest;
+}
+
+GType
+ephy_node_get_type (void)
+{
+	static GType type = 0;
+
+	if (G_UNLIKELY (type == 0))
+	{
+		type = g_boxed_type_register_static ("EphyNode",
+						     (GBoxedCopyFunc) ephy_node_ref,
+						     (GBoxedFreeFunc) ephy_node_unref);
+	}
+
+	return type;
+}
