@@ -16,7 +16,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: ephy-bookmarks.c,v 1.117 2005/10/16 20:29:26 chpe Exp $
+ *  $Id: ephy-bookmarks.c,v 1.121 2006/01/16 23:51:41 chpe Exp $
  */
 
 #include "config.h"
@@ -70,7 +70,6 @@ struct _EphyBookmarksPrivate
 	EphyNode *smartbookmarks;
 	EphyNode *lower_fav;
 	double lower_score;
-	GHashTable *props_dialogs;
 	guint disable_bookmark_editing_notifier_id;
 
 #ifdef ENABLE_ZEROCONF
@@ -95,7 +94,6 @@ static const EphyBookmarksBookmarkInfo default_bookmarks [] =
 	 */
 	{ N_("Search the web"), N_("http://www.google.com/search?q=%s&ie=UTF-8&oe=UTF-8") }
 };
-static int n_default_bookmarks = G_N_ELEMENTS (default_bookmarks);
 
 static const char *default_topics [] =
 {
@@ -106,7 +104,6 @@ static const char *default_topics [] =
 	N_("Travel"),
 	N_("Work")
 };
-static int n_default_topics = G_N_ELEMENTS (default_topics);
 
 /* Signals */
 enum
@@ -162,12 +159,12 @@ ephy_bookmarks_init_defaults (EphyBookmarks *eb)
 {
 	int i;
 
-	for (i = 0; i < n_default_topics; i++)
+	for (i = 0; i < G_N_ELEMENTS (default_topics); i++)
 	{
 		ephy_bookmarks_add_keyword (eb, _(default_topics[i]));
 	}
 
-	for (i = 0; i < n_default_bookmarks; i++)
+	for (i = 0; i < G_N_ELEMENTS (default_bookmarks); i++)
 	{
 		EphyNode *bmk;
 
@@ -470,12 +467,12 @@ redirect_cb (EphyHistory *history,
 				GTK_MESSAGE_QUESTION,
 				GTK_BUTTONS_NONE,
 				/* translators: the %s is the title of the bookmark */
-				_("Update bookmark \"%s\"?"),
+				_("Update bookmark “%s”?"),
 				title);
 	gtk_message_dialog_format_secondary_text
 		(GTK_MESSAGE_DIALOG (dialog),
 		/* translators: the %s is a URL */
-		_("The bookmarked page has moved to \"%s\"."),
+		_("The bookmarked page has moved to “%s”."),
 		to_uri);
 
 	gtk_dialog_add_button (GTK_DIALOG (dialog),
@@ -863,9 +860,6 @@ ephy_bookmarks_init (EphyBookmarks *eb)
 					       "bookmarks.rdf",
 					       NULL);
 
-	eb->priv->props_dialogs = g_hash_table_new (g_direct_hash,
-						    g_direct_equal);
-
 	/* Bookmarks */
 	eb->priv->bookmarks = ephy_node_new_with_id (db, BOOKMARKS_NODE_ID);
 	g_value_init (&value, G_TYPE_STRING);
@@ -1018,8 +1012,6 @@ ephy_bookmarks_finalize (GObject *object)
 
 	eel_gconf_notification_remove (eb->priv->disable_bookmark_editing_notifier_id);
 
-	g_hash_table_destroy (eb->priv->props_dialogs);
-
 	if (eb->priv->save_timeout_id != 0)
 	{
 		g_source_remove (eb->priv->save_timeout_id);
@@ -1100,25 +1092,43 @@ ephy_bookmarks_add (EphyBookmarks *eb,
 		    const char *title,
 		    const char *url)
 {
+	EphyHistory *history;
 	EphyNode *bm;
 	GValue value = { 0, };
 
 	bm = ephy_node_new (eb->priv->db);
 
 	if (bm == NULL) return NULL;
-
-	g_value_init (&value, G_TYPE_STRING);
-	g_value_set_string (&value, title);
-	ephy_node_set_property (bm, EPHY_NODE_BMK_PROP_TITLE,
-				&value);
-	g_value_unset (&value);
-
+	
+	if (url == NULL) return NULL;
 	g_value_init (&value, G_TYPE_STRING);
 	g_value_set_string (&value, url);
-	ephy_node_set_property (bm, EPHY_NODE_BMK_PROP_LOCATION,
-				&value);
+	ephy_node_set_property (bm, EPHY_NODE_BMK_PROP_LOCATION, &value);
 	g_value_unset (&value);
 
+	if (title == NULL || title[0] == '\0')
+	{
+		title = _("Untitled");
+	}	
+	g_value_init (&value, G_TYPE_STRING);
+	g_value_set_string (&value, title);
+	ephy_node_set_property (bm, EPHY_NODE_BMK_PROP_TITLE, &value);
+	g_value_unset (&value);
+
+	history = EPHY_HISTORY (ephy_embed_shell_get_global_history (embed_shell));
+	if (history != NULL)
+	{
+		const char *icon = ephy_history_get_icon (history, url);
+		if (icon != NULL)
+		{
+			g_value_init (&value, G_TYPE_STRING);
+			g_value_set_string (&value, icon);
+			ephy_node_set_property (bm, EPHY_NODE_BMK_PROP_ICON,
+						&value);
+			g_value_unset (&value);
+		}
+	}
+
 	update_has_smart_address (eb, bm, url);
 	update_bookmark_keywords (eb, bm);
 
@@ -1196,6 +1206,28 @@ ephy_bookmarks_set_icon	(EphyBookmarks *eb,
 	g_value_unset (&value);
 }
 
+
+void
+ephy_bookmarks_set_usericon (EphyBookmarks *eb,
+			     const char *url,
+			     const char *icon)
+{
+	EphyNode *node;
+	GValue value = { 0, };
+
+	g_return_if_fail (icon != NULL);
+
+	node = ephy_bookmarks_find_bookmark (eb, url);
+	if (node == NULL) return;
+
+	g_value_init (&value, G_TYPE_STRING);
+	g_value_set_string (&value, icon);
+	ephy_node_set_property (node, EPHY_NODE_BMK_PROP_USERICON,
+				&value);
+	g_value_unset (&value);
+}
+
+
 /* name must end with '=' */
 static char *
 get_option (char *start,
@@ -1356,57 +1388,6 @@ ephy_bookmarks_add_keyword (EphyBookmarks *eb,
 	return key;
 }
 
-
-static void
-prop_dialog_destroy_cb (EphyBookmarkProperties *dialog,
-			EphyBookmarks *bookmarks)
-{
-	EphyNode *bookmark;
-
-	bookmark = ephy_bookmark_properties_get_node (dialog);
-
-	g_hash_table_remove (bookmarks->priv->props_dialogs, bookmark);
-}
-
-static void
-bookmark_destroyed_cb (EphyNode *bookmark,
-		       GtkWidget *dialog)
-{
-	gtk_widget_destroy (dialog);
-}		 
-
-GtkWidget *
-ephy_bookmarks_show_bookmark_properties (EphyBookmarks *bookmarks,
-					 EphyNode *bookmark,
-					 GtkWidget *parent)
-{
-	GtkWidget *dialog = NULL;
-
-	g_return_val_if_fail (EPHY_IS_BOOKMARKS (bookmarks), NULL);
-	g_return_val_if_fail (EPHY_IS_NODE (bookmark), NULL);
-
-	dialog = g_hash_table_lookup (bookmarks->priv->props_dialogs, bookmark);
-
-	if (dialog == NULL)
-	{
-		dialog = ephy_bookmark_properties_new (bookmarks, bookmark, parent);
-
-		ephy_node_signal_connect_object (bookmark,
-						 EPHY_NODE_DESTROY,
-						 (EphyNodeCallback) bookmark_destroyed_cb,
-						 G_OBJECT (dialog));
-
-		g_signal_connect (dialog, "destroy",
-				  G_CALLBACK (prop_dialog_destroy_cb), bookmarks);
-		g_hash_table_insert (bookmarks->priv->props_dialogs,
-				     bookmark, dialog);
-	}
-
-	gtk_window_present (GTK_WINDOW (dialog));
-
-	return dialog;
-}
-
 void
 ephy_bookmarks_remove_keyword (EphyBookmarks *eb,
 			       EphyNode *keyword)
@@ -1589,13 +1570,15 @@ ephy_bookmarks_get_favorites (EphyBookmarks *eb)
 	return eb->priv->favorites;
 }
 
-#ifdef ENABLE_ZEROCONF
 EphyNode *
 ephy_bookmarks_get_local (EphyBookmarks *eb)
 {
+#ifdef ENABLE_ZEROCONF
 	return eb->priv->local;
-}
+#else
+	return NULL;
 #endif
+}
 
 EphyNode *
 ephy_bookmarks_get_not_categorized (EphyBookmarks *eb)
