@@ -16,7 +16,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: ephy-bookmarks.h,v 1.31 2005/01/08 19:41:47 chpe Exp $
+ *  $Id: ephy-bookmarks.h,v 1.37 2006/01/16 23:03:27 paharvey Exp $
  */
 
 #ifndef EPHY_BOOKMARKS_H
@@ -42,15 +42,17 @@ typedef struct _EphyBookmarksPrivate	EphyBookmarksPrivate;
 
 #define	BOOKMARKS_HIERARCHY_SEP	"->"
 
-enum
+typedef enum
 {
 	EPHY_NODE_BMK_PROP_TITLE	= 2,
 	EPHY_NODE_BMK_PROP_LOCATION	= 3,
 	EPHY_NODE_BMK_PROP_KEYWORDS	= 4,
 	EPHY_NODE_KEYWORD_PROP_NAME	= 5,
+	EPHY_NODE_BMK_PROP_USERICON	= 6,
 	EPHY_NODE_BMK_PROP_ICON		= 7,
-	EPHY_NODE_KEYWORD_PROP_PRIORITY	= 8
-};
+	EPHY_NODE_KEYWORD_PROP_PRIORITY	= 8,
+	EPHY_NODE_BMK_PROP_IMMUTABLE	= 15
+} EphyBookmarkProperty;
 
 struct _EphyBookmarks
 {
@@ -65,6 +67,9 @@ struct _EphyBookmarksClass
 	GObjectClass parent_class;
 
 	void	(* tree_changed)	(EphyBookmarks *eb);
+	char *	(* resolve_address)	(EphyBookmarks *eb,
+					 const char *address,
+					 const char *argument);
 };
 
 GType		  ephy_bookmarks_get_type		(void);
@@ -74,8 +79,6 @@ EphyBookmarks    *ephy_bookmarks_new			(void);
 EphyNode	 *ephy_bookmarks_get_from_id		(EphyBookmarks *eb,
 							 long id);
 
-GObject		 *ephy_bookmarks_get_toolbars_model	(EphyBookmarks *eb);
-
 /* Bookmarks */
 
 EphyNode	 *ephy_bookmarks_add			(EphyBookmarks *eb,
@@ -89,18 +92,20 @@ void		  ephy_bookmarks_set_icon		(EphyBookmarks *eb,
 							 const char *url,
 							 const char *icon);
 
+void		  ephy_bookmarks_set_usericon		(EphyBookmarks *eb,
+							 const char *url,
+							 const char *icon);
+
 void		  ephy_bookmarks_set_address    	(EphyBookmarks *eb,
 							 EphyNode *bookmark,
 							 const char *address);
 
-char		 *ephy_bookmarks_solve_smart_url	 (EphyBookmarks *eb,
-							 const char *smart_url,
-							 const char *content);
+char		 *ephy_bookmarks_resolve_address	(EphyBookmarks *eb,
+							 const char *address,
+							 const char *parameter);
+
+guint		ephy_bookmarks_get_smart_bookmark_width (EphyNode *bookmark);
 
-GtkWidget      *ephy_bookmarks_show_bookmark_properties (EphyBookmarks *bookmarks,
-							 EphyNode *bookmark,
-							 GtkWidget *parent_window);
-							
 
 /* Keywords */
 
@@ -143,6 +148,14 @@ EphyNode	 *ephy_bookmarks_get_not_categorized	(EphyBookmarks *eb);
 
 EphyNode	 *ephy_bookmarks_get_smart_bookmarks	(EphyBookmarks *eb);
 
+EphyNode	 *ephy_bookmarks_get_local		(EphyBookmarks *eb);
+
+/* Comparison functions, useful for sorting lists and arrays. */
+int            ephy_bookmarks_compare_topics            (gconstpointer a, gconstpointer b);
+int            ephy_bookmarks_compare_topic_pointers    (gconstpointer a, gconstpointer b);
+int            ephy_bookmarks_compare_bookmarks         (gconstpointer a, gconstpointer b);
+int            ephy_bookmarks_compare_bookmark_pointers (gconstpointer a, gconstpointer b);
+
 G_END_DECLS
 
 #endif
