@@ -16,7 +16,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: ephy-history-window.c,v 1.97.4.1 2005/10/31 18:52:07 chpe Exp $
+ *  $Id: ephy-history-window.c,v 1.101 2006/01/16 23:02:50 paharvey Exp $
  */
 
 #include "config.h"
@@ -56,7 +56,6 @@
 #include "window-commands.h"
 #include "ephy-file-helpers.h"
 #include "ephy-debug.h"
-#include "ephy-new-bookmark.h"
 #include "ephy-stock-icons.h"
 #include "ephy-gui.h"
 #include "ephy-stock-icons.h"
@@ -67,6 +66,7 @@
 #include "ephy-node.h"
 #include "ephy-node-common.h"
 #include "ephy-node-view.h"
+#include "ephy-bookmarks-ui.h"
 
 static const GtkTargetEntry page_drag_types [] =
 {
@@ -475,14 +475,8 @@ static void
 cmd_bookmark_link (GtkAction *action,
                    EphyHistoryWindow *editor)
 {
-        GtkWindow *window;
-        EphyBookmarks *bookmarks;
-        GtkWidget *new_bookmark;
         GList *selection;
 
-	window = GTK_WINDOW (editor);
-        bookmarks = ephy_shell_get_bookmarks (ephy_shell);
-
         selection = ephy_node_view_get_selection (EPHY_NODE_VIEW (editor->priv->pages_view));
 
 	if (g_list_length (selection) == 1)
@@ -494,16 +488,10 @@ cmd_bookmark_link (GtkAction *action,
 		node = selection->data;
 		location = ephy_node_get_property_string (node, EPHY_NODE_PAGE_PROP_LOCATION);
 		title = ephy_node_get_property_string (node, EPHY_NODE_PAGE_PROP_TITLE);
-		if (ephy_new_bookmark_is_unique (bookmarks, GTK_WINDOW (window),
-						 location))
-		{
-			new_bookmark = ephy_new_bookmark_new
-				(bookmarks, window, location);
-			ephy_new_bookmark_set_title
-				(EPHY_NEW_BOOKMARK (new_bookmark), title);
-			gtk_widget_show (new_bookmark);
-		}
+
+		ephy_bookmarks_ui_add_bookmark (GTK_WIDGET (editor), location, title);
 	}
+
 	g_list_free (selection);
 }
 
@@ -622,7 +610,7 @@ ephy_history_window_class_init (EphyHistoryWindowClass *klass)
 							      "Global history",
 							      "Global History",
 							      EPHY_TYPE_HISTORY,
-							      G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
+							      G_PARAM_READWRITE | G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_CONSTRUCT_ONLY));
 
 	g_type_class_add_private (object_class, sizeof(EphyHistoryWindowPrivate));
 }
@@ -1039,7 +1027,7 @@ build_search_box (EphyHistoryWindow *editor)
 	combo = gtk_combo_box_new_text ();
 	gtk_widget_show (combo);
 
-	gtk_combo_box_append_text (GTK_COMBO_BOX (combo), _("Last 30 Minutes"));
+	gtk_combo_box_append_text (GTK_COMBO_BOX (combo), _("Last 30 minutes"));
 	gtk_combo_box_append_text (GTK_COMBO_BOX (combo), _("Today"));
 	str = g_strdup_printf (ngettext ("Last %d day", "Last %d days", 2), 2);
 	gtk_combo_box_append_text (GTK_COMBO_BOX (combo), str);
