@@ -18,7 +18,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: ephy-tab.c,v 1.158.2.5 2005/11/27 13:53:56 chpe Exp $
+ *  $Id: ephy-tab.c,v 1.168 2005/11/18 18:37:36 chpe Exp $
  */
 
 #include "config.h"
@@ -70,7 +70,6 @@
 
 #define EPHY_TAB_GET_PRIVATE(object)(G_TYPE_INSTANCE_GET_PRIVATE ((object), EPHY_TYPE_TAB, EphyTabPrivate))
 
-#define MAX_HIDDEN_POPUPS 5
 #define MAX_HIDDEN_POPUPS	5
 #define RELOAD_DELAY		250 /* ms */
 #define RELOAD_DELAY_MAX_TICKS	40  /* RELOAD_DELAY * RELOAD_DELAY_MAX_TICKS = 10 s */
@@ -142,6 +141,7 @@ enum
 typedef struct
 {
 	char *url;
+	char *name;
 	char *features;
 } PopupInfo;
 
@@ -384,7 +384,7 @@ ephy_tab_class_init (EphyTabClass *class)
 							      "Address",
 							      "The tab's address",
 							      "",
-							      G_PARAM_READABLE));
+							      G_PARAM_READABLE | G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB));
 
 	g_object_class_install_property (object_class,
 					 PROP_DOCUMENT_TYPE,
@@ -393,7 +393,7 @@ ephy_tab_class_init (EphyTabClass *class)
 							    "The tab's documen type",
 							    EPHY_TYPE_EMBED_DOCUMENT_TYPE,
 							    EPHY_EMBED_DOCUMENT_HTML,
-							    G_PARAM_READABLE));
+							    G_PARAM_READABLE | G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB));
 
 	g_object_class_install_property (object_class,
 					 PROP_ICON,
@@ -401,7 +401,7 @@ ephy_tab_class_init (EphyTabClass *class)
 							      "Icon",
 							      "The tab icon's",
 							      GDK_TYPE_PIXBUF,
-							      G_PARAM_READABLE));
+							      G_PARAM_READABLE | G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB));
 
 	g_object_class_install_property (object_class,
 					 PROP_ICON_ADDRESS,
@@ -409,7 +409,7 @@ ephy_tab_class_init (EphyTabClass *class)
 							      "Icon address",
 							      "The tab icon's address",
 							      NULL,
-							      (G_PARAM_READABLE | G_PARAM_WRITABLE)));
+							      (G_PARAM_READWRITE | G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB)));
 
 	g_object_class_install_property (object_class,
 					 PROP_LOAD_PROGRESS,
@@ -419,7 +419,7 @@ ephy_tab_class_init (EphyTabClass *class)
 							   0,
 							   100,
 							   0,
-							   G_PARAM_READABLE));
+							   G_PARAM_READABLE | G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB));
 
 	g_object_class_install_property (object_class,
 					 PROP_LOAD_STATUS,
@@ -427,7 +427,7 @@ ephy_tab_class_init (EphyTabClass *class)
 							       "Load status",
 							       "The tab's load status",
 							       FALSE,
-							       G_PARAM_READABLE));
+							       G_PARAM_READABLE | G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB));
 
 	g_object_class_install_property (object_class,
 					 PROP_MESSAGE,
@@ -435,7 +435,7 @@ ephy_tab_class_init (EphyTabClass *class)
 							      "Message",
 							      "The tab's statusbar message",
 							      NULL,
-							      G_PARAM_READABLE));
+							      G_PARAM_READABLE | G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB));
 
 	g_object_class_install_property (object_class,
 					 PROP_NAVIGATION,
@@ -444,7 +444,7 @@ ephy_tab_class_init (EphyTabClass *class)
 							     "The tab's navigation flags",
 							     EPHY_TYPE_TAB_NAVIGATION_FLAGS,
 							     0,
-							     G_PARAM_READABLE));
+							     G_PARAM_READABLE | G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB));
 
 	g_object_class_install_property (object_class,
 					 PROP_SECURITY,
@@ -453,7 +453,7 @@ ephy_tab_class_init (EphyTabClass *class)
 							    "The tab's security level",
 							    EPHY_TYPE_EMBED_SECURITY_LEVEL,
 							    EPHY_EMBED_STATE_IS_UNKNOWN,
-							     G_PARAM_READABLE));
+							     G_PARAM_READABLE | G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB));
 
 	g_object_class_install_property (object_class,
 					 PROP_HIDDEN_POPUP_COUNT,
@@ -463,7 +463,7 @@ ephy_tab_class_init (EphyTabClass *class)
 							    0,
 							    G_MAXINT,
 							    0,
-							    G_PARAM_READABLE));
+							    G_PARAM_READABLE | G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB));
 
 	g_object_class_install_property (object_class,
 					 PROP_POPUPS_ALLOWED,
@@ -471,7 +471,7 @@ ephy_tab_class_init (EphyTabClass *class)
 						 	       "Popups Allowed",
 							       "Whether popup windows are to be displayed",
 							       FALSE,
-							       G_PARAM_READABLE | G_PARAM_WRITABLE));
+							       G_PARAM_READWRITE | G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB));
 
 	g_object_class_install_property (object_class,
 					 PROP_TITLE,
@@ -479,7 +479,7 @@ ephy_tab_class_init (EphyTabClass *class)
 							      "Title",
 							      "The tab's title",
 							      _("Blank page"),
-							      G_PARAM_READABLE));
+							      G_PARAM_READABLE | G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB));
 
 	g_object_class_install_property (object_class,
 					 PROP_TYPED_ADDRESS,
@@ -487,7 +487,7 @@ ephy_tab_class_init (EphyTabClass *class)
 							      "Typed Address",
 							      "The typed address",
 							      "",
-							      G_PARAM_READABLE | G_PARAM_WRITABLE));
+							      G_PARAM_READWRITE | G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB));
 
 	g_object_class_install_property (object_class,
 					 PROP_VISIBLE,
@@ -495,7 +495,7 @@ ephy_tab_class_init (EphyTabClass *class)
 							       "Visibility",
 							       "The tab's visibility",
 							       TRUE,
-							       G_PARAM_READABLE));
+							       G_PARAM_READABLE | G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB));
 
 	g_object_class_install_property (object_class,
 					 PROP_ZOOM,
@@ -505,7 +505,7 @@ ephy_tab_class_init (EphyTabClass *class)
 							     ZOOM_MINIMAL,
 							     ZOOM_MAXIMAL,
 							     1.0,
-							     G_PARAM_READABLE));
+							     G_PARAM_READABLE | G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB));
 
 	g_type_class_add_private (object_class, sizeof (EphyTabPrivate));
 }
@@ -514,6 +514,7 @@ static void
 popups_manager_free_info (PopupInfo *popup)
 {
 	g_free (popup->url);
+	g_free (popup->name);
 	g_free (popup->features);
 	g_free (popup);
 }
@@ -521,8 +522,10 @@ popups_manager_free_info (PopupInfo *popup)
 static void
 popups_manager_add (EphyTab *tab,
 		    const char *url,
+		    const char *name,
 		    const char *features)
 {
+	EphyTabPrivate *priv = tab->priv;
 	PopupInfo *popup;
 
 	LOG ("popups_manager_add: tab %p, url %s, features %s",
@@ -533,10 +536,10 @@ popups_manager_add (EphyTab *tab,
 	popup = g_new0 (PopupInfo, 1);
 
 	popup->url = g_strdup (url);
+	popup->name = g_strdup (name);
 	popup->features = g_strdup (features);
 
-	tab->priv->hidden_popups = g_slist_prepend
-		(tab->priv->hidden_popups, popup);
+	priv->hidden_popups = g_slist_prepend (priv->hidden_popups, popup);
 
 	if (popup_blocker_n_hidden (tab) > MAX_HIDDEN_POPUPS) /* bug #160863 */
 	{
@@ -653,8 +656,8 @@ popups_manager_show (PopupInfo *popup,
 	single = EPHY_EMBED_SINGLE
 		(ephy_embed_shell_get_embed_single (embed_shell));
 
-	ephy_embed_single_open_window (single, embed, popup->url, "",
-				       popup->features);
+	ephy_embed_single_open_window (single, embed, popup->url,
+				       popup->name, popup->features);
 
 	popups_manager_free_info (popup);
 }
@@ -712,7 +715,7 @@ popups_manager_hide (EphyWindow *window,
 
 	features = popups_manager_new_window_info (window);
 
-	popups_manager_add (parent_tab, location, features);
+	popups_manager_add (parent_tab, location, "" /* FIXME? maybe _blank? */, features);
 
 	gtk_widget_destroy (GTK_WIDGET (window));
 
@@ -1645,15 +1648,15 @@ update_net_state_message (EphyTab *tab, const char *uri, EphyEmbedNetState flags
 	{
 		if (flags & EPHY_EMBED_STATE_REDIRECTING)
 		{
-			msg = _("Redirecting to %s...");
+			msg = _("Redirecting to “%s”...");
 		}
 		else if (flags & EPHY_EMBED_STATE_TRANSFERRING)
 		{
-			msg = _("Transferring data from %s...");
+			msg = _("Transferring data from “%s”...");
 		}
 		else if (flags & EPHY_EMBED_STATE_NEGOTIATING)
 		{
-			msg = _("Waiting for authorization from %s...");
+			msg = _("Waiting for authorization from “%s”...");
 		}
 	}
 
@@ -1661,7 +1664,7 @@ update_net_state_message (EphyTab *tab, const char *uri, EphyEmbedNetState flags
 	{
 		if (flags & EPHY_EMBED_STATE_START)
 		{
-			msg = _("Loading %s...");
+			msg = _("Loading “%s”...");
 		}
 	}
 
@@ -1799,10 +1802,13 @@ ephy_tab_new_window_cb (EphyEmbed *embed,
 }
 
 static void
-ephy_tab_popup_blocked_cb (EphyEmbed *embed, const char *url,
-			   const char *features, EphyTab *tab)
+ephy_tab_popup_blocked_cb (EphyEmbed *embed,
+			   const char *url,
+			   const char *name,
+			   const char *features,
+			   EphyTab *tab)
 {
-	popups_manager_add (tab, url, features);
+	popups_manager_add (tab, url, name, features);
 }
 
 static void
@@ -1836,8 +1842,19 @@ open_link_in_new_tab (EphyTab *tab,
 
 	if (new_tab)
 	{
-		return ephy_link_open (EPHY_LINK (tab), link_address,
-				       tab, EPHY_LINK_NEW_TAB) != NULL;
+		EphyTab *dest;
+		dest = ephy_link_open (EPHY_LINK (tab), link_address,
+				       tab, EPHY_LINK_NEW_TAB);
+
+		if (dest)
+		{
+			ephy_embed_shistory_copy (ephy_tab_get_embed (tab),
+						  ephy_tab_get_embed (dest),
+						  TRUE,   /* back history */
+						  FALSE,  /* forward history */
+					  	  FALSE); /* current index */
+			return TRUE;
+		}
 	}
 
 	return FALSE;
@@ -2195,7 +2212,7 @@ ephy_tab_set_title (EphyTab *tab,
 {
 	EphyTabPrivate *priv = tab->priv;
 
-	if (!priv->is_blank && (title == NULL || title[0] == '\0'))
+	if (!priv->is_blank && (title == NULL || g_strstrip (title)[0] == '\0'))
 	{
 
 		g_free (title);
@@ -2209,9 +2226,14 @@ ephy_tab_set_title (EphyTab *tab,
 			priv->is_blank = TRUE;
 		}
 	}
+	else if (priv->is_blank && title != NULL)
+	{
+		g_free (title);
+		title = NULL;
+	}
 
 	g_free (priv->title);
-	priv->title = title;
+	priv->title = ephy_string_shorten (title, MAX_TITLE_LENGTH);
 
 	g_object_notify (G_OBJECT (tab), "title");
 }
