@@ -15,19 +15,19 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: popup-commands.c,v 1.52 2005/04/19 09:37:13 chpe Exp $
+ *  $Id: popup-commands.c,v 1.57 2006/01/16 23:51:41 chpe Exp $
  */
 
 #include "config.h"
 
 #include "popup-commands.h"
 #include "ephy-shell.h"
-#include "ephy-new-bookmark.h"
 #include "ephy-embed-factory.h"
 #include "ephy-embed-persist.h"
 #include "ephy-prefs.h"
 #include "eel-gconf-extensions.h"
 #include "ephy-file-helpers.h"
+#include "ephy-bookmarks-ui.h"
 
 #include <string.h>
 #include <glib/gi18n.h>
@@ -38,31 +38,20 @@
 #include <libgnomevfs/gnome-vfs-ops.h>
 #include <libgnomevfs/gnome-vfs-mime-handlers.h>
 
-static EphyEmbedEvent *
-get_event_info (EphyWindow *window)
-{
-	EphyEmbedEvent *info;
-
-	info = EPHY_EMBED_EVENT (g_object_get_data
-		(G_OBJECT (window), "context_event"));
-	g_return_val_if_fail (info != NULL, NULL);
-
-	return info;
-}
-
 void
 popup_cmd_link_in_new_window (GtkAction *action,
 		              EphyWindow *window)
 {
-	EphyEmbedEvent *info;
+	EphyEmbedEvent *event;
 	EphyTab *tab;
 	const GValue *value;
 
 	tab = ephy_window_get_active_tab (window);
 
-	info = get_event_info (window);
+	event = ephy_window_get_context_event (window);
+	g_return_if_fail (event != NULL);
 
-	ephy_embed_event_get_property (info, "link", &value);
+	ephy_embed_event_get_property (event, "link", &value);
 
 	ephy_shell_new_tab (ephy_shell, NULL, tab,
 			    g_value_get_string (value),
@@ -74,15 +63,16 @@ void
 popup_cmd_link_in_new_tab (GtkAction *action,
 		           EphyWindow *window)
 {
-	EphyEmbedEvent *info;
+	EphyEmbedEvent *event;
 	EphyTab *tab;
 	const GValue *value;
 
 	tab = ephy_window_get_active_tab (window);
 
-	info = get_event_info (window);
+	event = ephy_window_get_context_event (window);
+	g_return_if_fail (event != NULL);
 
-	ephy_embed_event_get_property (info, "link", &value);
+	ephy_embed_event_get_property (event, "link", &value);
 
 	ephy_shell_new_tab (ephy_shell, window, tab,
 			    g_value_get_string (value),
@@ -94,9 +84,7 @@ void
 popup_cmd_bookmark_link (GtkAction *action,
 			 EphyWindow *window)
 {
-	GtkWidget *new_bookmark;
-	EphyBookmarks *bookmarks;
-	EphyEmbedEvent *info;
+	EphyEmbedEvent *event;
 	const GValue *link_title;
 	const GValue *link_rel;
 	const GValue *link;
@@ -107,13 +95,14 @@ popup_cmd_bookmark_link (GtkAction *action,
 	const char *rel;
 	gboolean is_smart;
 
-	info = get_event_info (window);
+	event = ephy_window_get_context_event (window);
+	g_return_if_fail (event != NULL);
 
-	ephy_embed_event_get_property (info, "link_is_smart", &link_is_smart);
-	ephy_embed_event_get_property (info, "link", &link);
-	ephy_embed_event_get_property (info, "link_title", &link_title);
-	ephy_embed_event_get_property (info, "link_rel", &link_rel);
-	ephy_embed_event_get_property (info, "linktext", &linktext);
+	ephy_embed_event_get_property (event, "link_is_smart", &link_is_smart);
+	ephy_embed_event_get_property (event, "link", &link);
+	ephy_embed_event_get_property (event, "link_title", &link_title);
+	ephy_embed_event_get_property (event, "link_rel", &link_rel);
+	ephy_embed_event_get_property (event, "linktext", &linktext);
 
 	location = g_value_get_string (link);
 	g_return_if_fail (location);
@@ -132,18 +121,13 @@ popup_cmd_bookmark_link (GtkAction *action,
 	{
 		title = location;
 	}
-
-	bookmarks = ephy_shell_get_bookmarks (ephy_shell);
-	if (ephy_new_bookmark_is_unique (bookmarks, GTK_WINDOW (window),
-					 location))
+	
+	if (is_smart)
 	{
-		new_bookmark = ephy_new_bookmark_new
-			(bookmarks, GTK_WINDOW (window),
-			 is_smart ? rel : location);
-		ephy_new_bookmark_set_title
-			(EPHY_NEW_BOOKMARK (new_bookmark), title);
-		gtk_widget_show (new_bookmark);
+		location = rel;
 	}
+
+	ephy_bookmarks_ui_add_bookmark (GTK_WIDGET (window), location, title);
 }
 
 static void
@@ -164,8 +148,8 @@ popup_cmd_copy_link_address (GtkAction *action,
 	const char *address;
 	const GValue *value;
 
-	event = get_event_info (window);
-	g_return_if_fail (EPHY_IS_EMBED_EVENT (event));
+	event = ephy_window_get_context_event (window);
+	g_return_if_fail (event != NULL);
 
 	context = ephy_embed_event_get_context (event);
 
@@ -190,27 +174,29 @@ save_property_url (GtkAction *action,
 		   gboolean ask_dest,
 		   const char *property)
 {
-	EphyEmbedEvent *info;
+	EphyEmbedEvent *event;
 	const char *location;
 	const GValue *value;
 	EphyEmbedPersist *persist;
 	EphyEmbed *embed;
 
+	event = ephy_window_get_context_event (window);
+	g_return_if_fail (event != NULL);
+
 	embed = ephy_window_get_active_embed (window);
 	g_return_if_fail (embed != NULL);
 
-	info = get_event_info (window);
-	ephy_embed_event_get_property (info, property, &value);
+	ephy_embed_event_get_property (event, property, &value);
 	location = g_value_get_string (value);
 
 	persist = EPHY_EMBED_PERSIST
 		(ephy_embed_factory_new_object (EPHY_TYPE_EMBED_PERSIST));
 
-	ephy_embed_persist_set_embed (persist, embed);
 	ephy_embed_persist_set_fc_title (persist, title);
 	ephy_embed_persist_set_fc_parent (persist, GTK_WINDOW (window));
 	ephy_embed_persist_set_flags
-		(persist, ask_dest ? EPHY_EMBED_PERSIST_ASK_DESTINATION : 0);
+		(persist, EPHY_EMBED_PERSIST_FROM_CACHE |
+			  (ask_dest ? EPHY_EMBED_PERSIST_ASK_DESTINATION : 0));
 	ephy_embed_persist_set_persist_key
 		(persist, CONF_STATE_SAVE_DIR);
 	ephy_embed_persist_set_source (persist, location);
@@ -224,7 +210,7 @@ void
 popup_cmd_open_link (GtkAction *action,
 		     EphyWindow *window)
 {
-	EphyEmbedEvent *info;
+	EphyEmbedEvent *event;
 	const char *location;
 	const GValue *value;
 	EphyEmbed *embed;
@@ -232,8 +218,8 @@ popup_cmd_open_link (GtkAction *action,
 	embed = ephy_window_get_active_embed (window);
 	g_return_if_fail (embed != NULL);
 
-	info = get_event_info (window);
-	ephy_embed_event_get_property (info, "link", &value);
+	event = ephy_window_get_context_event (window);
+	ephy_embed_event_get_property (event, "link", &value);
 	location = g_value_get_string (value);
 
 	ephy_embed_load_url (embed, location);
@@ -293,18 +279,20 @@ void
 popup_cmd_set_image_as_background (GtkAction *action,
 				   EphyWindow *window)
 {
-	EphyEmbedEvent *info;
+	EphyEmbedEvent *event;
 	const char *location;
 	char *dest, *base, *base_converted;
 	const GValue *value;
 	EphyEmbedPersist *persist;
 	EphyEmbed *embed;
 
+	event = ephy_window_get_context_event (window);
+	g_return_if_fail (event != NULL);
+
 	embed = ephy_window_get_active_embed (window);
 	g_return_if_fail (embed != NULL);
 
-	info = get_event_info (window);
-	ephy_embed_event_get_property (info, "image", &value);
+	ephy_embed_event_get_property (event, "image", &value);
 	location = g_value_get_string (value);
 
 	persist = EPHY_EMBED_PERSIST
@@ -314,9 +302,9 @@ popup_cmd_set_image_as_background (GtkAction *action,
 	base_converted = g_filename_from_utf8 (base, -1, NULL, NULL, NULL);
 	dest = g_build_filename (ephy_dot_dir (), base_converted, NULL);
 
-	ephy_embed_persist_set_embed (persist, embed);
 	ephy_embed_persist_set_dest (persist, dest);
-	ephy_embed_persist_set_flags (persist, EPHY_EMBED_PERSIST_NO_VIEW);
+	ephy_embed_persist_set_flags (persist, EPHY_EMBED_PERSIST_NO_VIEW |
+				     	       EPHY_EMBED_PERSIST_FROM_CACHE);
 	ephy_embed_persist_set_source (persist, location);
 
 	g_signal_connect (persist, "completed",
@@ -334,24 +322,16 @@ void
 popup_cmd_copy_image_location (GtkAction *action,
 			       EphyWindow *window)
 {
-	EphyEmbedEvent *info;
+	EphyEmbedEvent *event;
 	const char *location;
 	const GValue *value;
 
-	info = get_event_info (window);
-	ephy_embed_event_get_property (info, "image", &value);
+	event = ephy_window_get_context_event (window);
+	ephy_embed_event_get_property (event, "image", &value);
 	location = g_value_get_string (value);
 	popup_cmd_copy_to_clipboard (window, location);
 }
 
-void
-popup_cmd_save_background_as (GtkAction *action,
-			      EphyWindow *window)
-{
-	save_property_url (action, _("Save Background As"),
-			   window, TRUE, "background_image");
-}
-
 void
 popup_cmd_open_frame (GtkAction *action,
 		      EphyWindow *window)
@@ -426,7 +406,8 @@ save_temp_source (const char *address)
 		(ephy_embed_factory_new_object (EPHY_TYPE_EMBED_PERSIST));
 
 	ephy_embed_persist_set_source (persist, address);
-	ephy_embed_persist_set_flags (persist, EPHY_EMBED_PERSIST_NO_VIEW);
+	ephy_embed_persist_set_flags (persist, EPHY_EMBED_PERSIST_FROM_CACHE |
+			 		       EPHY_EMBED_PERSIST_NO_VIEW);
 	ephy_embed_persist_set_dest (persist, tmp);
 
 	g_signal_connect (persist, "completed",
@@ -442,17 +423,19 @@ void
 popup_cmd_open_image (GtkAction *action,
 		      EphyWindow *window)
 {
-	EphyEmbedEvent *info;
+	EphyEmbedEvent *event;
 	const char *address;
 	char *scheme;
 	const GValue *value;
 	EphyEmbed *embed;
 
+	event = ephy_window_get_context_event (window);
+	g_return_if_fail (event != NULL);
+
 	embed = ephy_window_get_active_embed (window);
 	g_return_if_fail (embed != NULL);
 
-	info = get_event_info (window);
-	ephy_embed_event_get_property (info, "image", &value);
+	ephy_embed_event_get_property (event, "image", &value);
 	address = g_value_get_string (value);
 
 	scheme = gnome_vfs_get_uri_scheme (address);
