@@ -15,7 +15,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: window-commands.c,v 1.165 2005/08/01 19:28:53 chpe Exp $
+ *  $Id: window-commands.c,v 1.182 2006/01/16 23:51:41 chpe Exp $
  */
 
 #include "config.h"
@@ -34,7 +34,6 @@
 #include "ephy-dialog.h"
 #include "ephy-bookmarks-editor.h"
 #include "ephy-history-window.h"
-#include "ephy-new-bookmark.h"
 #include "ephy-file-chooser.h"
 #include "ephy-file-helpers.h"
 #include "ephy-toolbar.h"
@@ -45,6 +44,8 @@
 #include "ephy-toolbar-editor.h"
 #include "ephy-find-toolbar.h"
 #include "ephy-location-entry.h"
+#include "ephy-bookmarks-ui.h"
+#include "pdm-dialog.h"
 
 #include <string.h>
 #include <glib.h>
@@ -95,41 +96,12 @@ window_cmd_file_print (GtkAction *action,
 	ephy_embed_print (embed);
 }
 
-void
-window_cmd_go_back (GtkAction *action,
-		    EphyWindow *window)
-{
-	EphyEmbed *embed;
-
-	embed = ephy_window_get_active_embed (window);
-	g_return_if_fail (embed != NULL);
-
-	gtk_widget_grab_focus (GTK_WIDGET (embed));
-
-	ephy_embed_go_back (embed);
-}
-
-void
-window_cmd_go_up (GtkAction *action,
-		  EphyWindow *window)
-{
-	EphyEmbed *embed;
-
-	embed = ephy_window_get_active_embed (window);
-	g_return_if_fail (embed != NULL);
-
-	gtk_widget_grab_focus (GTK_WIDGET (embed));
-
-	ephy_embed_go_up (embed);
-}
-
 void
 window_cmd_file_send_to	(GtkAction *action,
 			 EphyWindow *window)
 {
 	EphyTab *tab;
 	EphyEmbed *embed;
-	const char *address;
 	char *url, *location, *title;
 
 	tab = ephy_window_get_active_tab (window);
@@ -138,20 +110,8 @@ window_cmd_file_send_to	(GtkAction *action,
 	embed = ephy_window_get_active_embed (window);
 	g_return_if_fail (embed != NULL);
 
-	address = ephy_tab_get_address (tab);
-	location = gnome_vfs_escape_string (address);
-
-	title = ephy_embed_get_title (embed);
-	if (title != NULL)
-	{
-		char *tmp = gnome_vfs_escape_string (title);
-		g_free (title);
-		title = tmp;
-	}
-	else
-	{
-		title = gnome_vfs_escape_string (_("Check this out!"));
-	}
+	location = gnome_vfs_escape_string (ephy_tab_get_address (tab));
+	title = gnome_vfs_escape_string (ephy_tab_get_title (tab));
 
 	url = g_strconcat ("mailto:",
                            "?Subject=", title,
@@ -164,18 +124,31 @@ window_cmd_file_send_to	(GtkAction *action,
 	g_free (url);
 }
 
-void
-window_cmd_go_forward (GtkAction *action,
-		       EphyWindow *window)
+static gboolean
+event_with_shift (void)
 {
-	EphyEmbed *embed;
+	GdkEvent *event;
+	GdkEventType type = 0;
+	guint state = 0;
 
-	embed = ephy_window_get_active_embed (window);
-	g_return_if_fail (embed != NULL);
+	event = gtk_get_current_event ();
+	if (event)
+	{
+		type = event->type;
 
-	gtk_widget_grab_focus (GTK_WIDGET (embed));
+		if (type == GDK_BUTTON_RELEASE)
+		{
+			state = event->button.state; 
+		}
+		else if (type == GDK_KEY_PRESS || type == GDK_KEY_RELEASE)
+		{
+			state = event->key.state;
+		}
 
-	ephy_embed_go_forward (embed);
+		gdk_event_free (event);
+	}
+
+	return (state & GDK_SHIFT_MASK) != 0;
 }
 
 void
@@ -204,39 +177,13 @@ window_cmd_view_reload (GtkAction *action,
 		        EphyWindow *window)
 {
 	EphyEmbed *embed;
-	GdkEvent *event;
-	GdkEventType type;
-	guint state = 0;
-	gboolean force = FALSE;
 
 	embed = ephy_window_get_active_embed (window);
 	g_return_if_fail (embed != NULL);
 
-	event = gtk_get_current_event ();
-	if (event)
-	{
-		type = event->type;
-
-		if (type == GDK_BUTTON_RELEASE)
-		{
-			state = event->button.state; 
-		}
-		else if (type == GDK_KEY_RELEASE)
-		{
-			state = event->key.state;
-		}
-
-		gdk_event_free (event);
-	}
-
-	if (state & GDK_SHIFT_MASK)
-	{
-		force = TRUE;
-	}
-
 	gtk_widget_grab_focus (GTK_WIDGET (embed));
 
-	ephy_embed_reload (embed, force);
+	ephy_embed_reload (embed, event_with_shift ());
 }
 
 void
@@ -286,10 +233,6 @@ window_cmd_file_bookmark_page (GtkAction *action,
 {
 	EphyTab *tab;
 	EphyEmbed *embed;
-	EphyBookmarks *bookmarks;
-	GtkWidget *new_bookmark;
-	const char *location, *icon;
-	char *title;
 
 	tab = ephy_window_get_active_tab (window);
 	g_return_if_fail (tab != NULL);
@@ -297,30 +240,9 @@ window_cmd_file_bookmark_page (GtkAction *action,
 	embed = ephy_window_get_active_embed (window);
 	g_return_if_fail (embed != NULL);
 
-	location = ephy_tab_get_address (tab);
-
-	title = ephy_embed_get_title (embed);
-	if (title == NULL)
-	{
-		title = g_strdup (_("Untitled"));
-	}
-
-	icon = ephy_tab_get_icon_address (tab);
-
-	bookmarks = ephy_shell_get_bookmarks (ephy_shell);
-	if (ephy_new_bookmark_is_unique (bookmarks, GTK_WINDOW (window),
-					 location))
-	{
-		new_bookmark = ephy_new_bookmark_new
-			(bookmarks, GTK_WINDOW (window), location);
-		ephy_new_bookmark_set_title
-			(EPHY_NEW_BOOKMARK (new_bookmark), title);
-		ephy_new_bookmark_set_icon
-			(EPHY_NEW_BOOKMARK (new_bookmark), icon);
-		gtk_widget_show (new_bookmark);
-	}
-
-	g_free (title);
+	ephy_bookmarks_ui_add_bookmark (GTK_WIDGET (window),
+					ephy_tab_get_address (tab),
+					ephy_tab_get_title (tab));
 }
 
 static void
@@ -402,14 +324,14 @@ window_cmd_file_work_offline (GtkAction *action,
 
 	single = EPHY_EMBED_SINGLE (ephy_embed_shell_get_embed_single (embed_shell));
 	offline = gtk_toggle_action_get_active (GTK_TOGGLE_ACTION (action));
-	ephy_embed_single_set_offline_mode (single, offline);
+	ephy_embed_single_set_network_status (single, !offline);
 }
 
 void
 window_cmd_file_close_window (GtkAction *action,
 		              EphyWindow *window)
 {
-	EphyTab *tab;
+	EphyEmbed *embed;
 
 	if (eel_gconf_get_boolean (CONF_LOCKDOWN_DISABLE_QUIT) &&
 	    gtk_notebook_get_n_pages (GTK_NOTEBOOK (ephy_window_get_notebook (window))) == 1)
@@ -417,10 +339,10 @@ window_cmd_file_close_window (GtkAction *action,
 		return;
 	}
 
-	tab = ephy_window_get_active_tab (window);
-	g_return_if_fail (tab != NULL);
+	embed = ephy_window_get_active_embed (window);
+	g_return_if_fail (embed != NULL);
 
-	ephy_window_remove_tab (window, tab);
+	ephy_embed_close (embed);
 }
 
 void
@@ -710,14 +632,45 @@ window_cmd_go_history (GtkAction *action,
 void
 window_cmd_edit_personal_data (GtkAction *action,
 		               EphyWindow *window)
+{
+	PdmDialog *dialog;
+	EphyTab *tab;
+	GnomeVFSURI *uri;
+	const char *host;
+
+	tab = ephy_window_get_active_tab (window);
+	if (tab == NULL) return;
+
+	uri = gnome_vfs_uri_new (ephy_tab_get_address (tab));
+
+	host = uri != NULL ? gnome_vfs_uri_get_host_name (uri) : NULL;
+
+	dialog = EPHY_PDM_DIALOG (ephy_shell_get_pdm_dialog (ephy_shell));
+	pdm_dialog_open (dialog, host);
+
+	if (uri != NULL)
+	{
+		gnome_vfs_uri_unref (uri);
+	}
+}
+
+#ifdef ENABLE_CERTIFICATE_MANAGER
+
+#include "ephy-cert-manager-dialog.h"
+
+void
+window_cmd_edit_certificates (GtkAction *action,
+			      EphyWindow *window)
 {
 	EphyDialog *dialog;
 
-	dialog = EPHY_DIALOG (ephy_shell_get_pdm_dialog (ephy_shell));
+	dialog = certs_manager_dialog_new ();
 
 	ephy_dialog_show (dialog);
 }
 
+#endif /* ENABLE_CERTIFICATE_MANAGER */
+
 void
 window_cmd_edit_prefs (GtkAction *action,
 		       EphyWindow *window)
@@ -759,6 +712,7 @@ window_cmd_help_about (GtkAction *action,
 		"",
 		_("Contributors:"),
 		"Crispin Flowerday",
+		"Peter Harvey",
 		"Raphaël Slinckx",
 		"",
 		_("Past developers:"),
@@ -771,15 +725,15 @@ window_cmd_help_about (GtkAction *action,
 		"David Bordoley",
 		"",
 		_("Contact us at:"),
-		"<epiphany-list@gnome.org> or <gnome-doc-list@gnome.org>",
+		_("<epiphany-list@gnome.org> or <gnome-doc-list@gnome.org>"),
 		NULL
 	};
 
 	gtk_show_about_dialog (GTK_WINDOW (window),
-			       "name", _("Epiphany Web Browser"),
+			       "name", _("GNOME Web Browser"),
 			       "version", VERSION,
 			       "copyright", "Copyright © 2002-2004 Marco Pesenti Gritti\n"
-					    "Copyright © 2003-2005 The Epiphany Developers",
+					    "Copyright © 2003-2006 The GNOME Web Browser Developers",
 			       "authors", authors,
 			       "documenters", documenters,
 				/* Translators: This is a special message that shouldn't be translated
@@ -793,6 +747,7 @@ window_cmd_help_about (GtkAction *action,
 			       "translator-credits", _("translator-credits"),
 			       "logo-icon-name", "web-browser",
 			       "website", "http://www.gnome.org/projects/epiphany",
+			       "website-label", _("GNOME Web Browser Website"),
 			       NULL);
 }
 
@@ -920,25 +875,3 @@ window_cmd_browse_with_caret (GtkAction *action,
 	active = gtk_toggle_action_get_active (GTK_TOGGLE_ACTION (action));
 	eel_gconf_set_boolean (CONF_BROWSE_WITH_CARET, active);
 }
-
-#ifdef HAVE_X11_XF86KEYSYM_H
-void
-window_cmd_go_home (GtkAction *action,
-		    EphyWindow *window)
-{
-	char *location;
-
-	location = eel_gconf_get_string (CONF_GENERAL_HOMEPAGE);
-
-	if (location == NULL || location[0] == '\0')
-	{
-		g_free (location);
-
-		location = g_strdup ("about:blank");
-	}
-
-	ephy_window_load_url (window, location);
-
-	g_free (location);
-}
-#endif /* HAVE_X11_XF86KEYSYM_H */
